@@ -7,6 +7,7 @@ __all__ = ['Profile',
            'MetroFirefoxProfile',
            'ThunderbirdProfile']
 
+import json
 import os
 import time
 import tempfile
@@ -20,100 +21,155 @@ from prefs import Preferences
 from shutil import copytree, rmtree
 from webapps import WebappCollection
 
-try:
-    import json
-except ImportError:
-    import simplejson as json
 
 class Profile(object):
-    """Handles all operations regarding profile. Created new profiles, installs extensions,
-    sets preferences and handles cleanup.
+    """Handles all operations regarding profile.
 
-    :param profile: Path to the profile
-    :param addons: String of one or list of addons to install
-    :param addon_manifests: Manifest for addons, see http://ahal.ca/blog/2011/bulk-installing-fx-addons/
-    :param apps: Dictionary or class of webapps to install
-    :param preferences: Dictionary or class of preferences
-    :param locations: ServerLocations object
-    :param proxy: setup a proxy
-    :param restore: If true remove all installed addons preferences when cleaning up
+    Creating new profiles, installing add-ons, setting preferences and
+    handling cleanup.
     """
 
     def __init__(self, profile=None, addons=None, addon_manifests=None, apps=None,
                  preferences=None, locations=None, proxy=None, restore=True):
+        """
+        :param profile: Path to the profile
+        :param addons: String of one or list of addons to install
+        :param addon_manifests: Manifest for addons (see http://bit.ly/17jQ7i6)
+        :param apps: Dictionary or class of webapps to install
+        :param preferences: Dictionary or class of preferences
+        :param locations: ServerLocations object
+        :param proxy: Setup a proxy
+        :param restore: Flag for removing all custom settings during cleanup
+        """
+        self._addons = addons
+        self._addon_manifests = addon_manifests
+        self._apps = apps
+        self._locations = locations
+        self._proxy = proxy
 
-        # if true, remove installed addons/prefs afterwards
-        self.restore = restore
+        # Prepare additional preferences
+        if preferences:
+            if isinstance(preferences, dict):
+                # unordered
+                preferences = preferences.items()
 
-        # prefs files written to
-        self.written_prefs = set()
-
-        # our magic markers
-        nonce = '%s %s' % (str(time.time()), uuid.uuid4())
-        self.delimeters = ('#MozRunner Prefs Start %s' % nonce,'#MozRunner Prefs End %s' % nonce)
+            # sanity check
+            assert not [i for i in preferences if len(i) != 2]
+        else:
+            preferences = []
+        self._preferences = preferences
 
         # Handle profile creation
         self.create_new = not profile
         if profile:
             # Ensure we have a full path to the profile
             self.profile = os.path.abspath(os.path.expanduser(profile))
-            if not os.path.exists(self.profile):
-                os.makedirs(self.profile)
         else:
-            self.profile = self.create_new_profile()
+            self.profile = tempfile.mkdtemp(suffix='.mozrunner')
 
-        # set preferences
+        self.restore = restore
+
+        # Initialize all class members
+        self._internal_init()
+
+    def _internal_init(self):
+        """Internal: Initialize all class members to their default value"""
+
+        if not os.path.exists(self.profile):
+            os.makedirs(self.profile)
+
+        # Preferences files written to
+        self.written_prefs = set()
+
+        # Our magic markers
+        nonce = '%s %s' % (str(time.time()), uuid.uuid4())
+        self.delimeters = ('#MozRunner Prefs Start %s' % nonce,
+                           '#MozRunner Prefs End %s' % nonce)
+
+        # If sub-classes want to set default preferences
         if hasattr(self.__class__, 'preferences'):
-            # class preferences
             self.set_preferences(self.__class__.preferences)
-        self._preferences = preferences
-        if preferences:
-            # supplied preferences
-            if isinstance(preferences, dict):
-                # unordered
-                preferences = preferences.items()
-            # sanity check
-            assert not [i for i in preferences
-                        if len(i) != 2]
-        else:
-            preferences = []
-        self.set_preferences(preferences)
+        # Set additional preferences
+        self.set_preferences(self._preferences)
 
-        # set permissions
-        self._locations = locations # store this for reconstruction
-        self._proxy = proxy
-        self.permissions = Permissions(self.profile, locations)
-        prefs_js, user_js = self.permissions.network_prefs(proxy)
+        self.permissions = Permissions(self.profile, self._locations)
+        prefs_js, user_js = self.permissions.network_prefs(self._proxy)
         self.set_preferences(prefs_js, 'prefs.js')
         self.set_preferences(user_js)
 
-        # handle addon installation
-        self.addon_manager = AddonManager(self.profile)
-        self.addon_manager.install_addons(addons, addon_manifests)
+        # handle add-on installation
+        self.addon_manager = AddonManager(self.profile, restore=self.restore)
+        self.addon_manager.install_addons(self._addons, self._addon_manifests)
 
         # handle webapps
-        self.webapps = WebappCollection(profile=self.profile, apps=apps)
+        self.webapps = WebappCollection(profile=self.profile, apps=self._apps)
         self.webapps.update_manifests()
 
-    def exists(self):
-        """returns whether the profile exists or not"""
-        return os.path.exists(self.profile)
+    def __del__(self):
+      self.cleanup()
+
+    ### cleanup
+
+    def cleanup(self):
+        """Cleanup operations for the profile."""
+
+        if self.restore:
+            # If copies of those class instances exist ensure we correctly
+            # reset them all (see bug 934484)
+            self.addon_manager.clean()
+            self.clean_preferences()
+            self.permissions.clean_db()
+            self.webapps.clean()
+
+            # If it's a temporary profile we have to remove it
+            if self.create_new:
+                if os.path.exists(self.profile):
+                    rmtree(self.profile, onerror=self._cleanup_error)
 
     def reset(self):
         """
         reset the profile to the beginning state
         """
         self.cleanup()
-        if self.create_new:
-            profile = None
-        else:
-            profile = self.profile
-        self.__init__(profile=profile,
-                      addons=self.addon_manager.installed_addons,
-                      addon_manifests=self.addon_manager.installed_manifests,
-                      preferences=self._preferences,
-                      locations=self._locations,
-                      proxy = self._proxy)
+
+        self._internal_init()
+
+    def _cleanup_error(self, function, path, excinfo):
+        """ Specifically for windows we need to handle the case where the windows
+            process has not yet relinquished handles on files, so we do a wait/try
+            construct and timeout if we can't get a clear road to deletion
+        """
+
+        try:
+            from exceptions import WindowsError
+            from time import sleep
+            def is_file_locked():
+                return excinfo[0] is WindowsError and excinfo[1].winerror == 32
+
+            if excinfo[0] is WindowsError and excinfo[1].winerror == 32:
+                # Then we're on windows, wait to see if the file gets unlocked
+                # we wait 10s
+                count = 0
+                while count < 10:
+                    sleep(1)
+                    try:
+                        function(path)
+                        break
+                    except:
+                        count += 1
+        except ImportError:
+            # We can't re-raise an error, so we'll hope the stuff above us will throw
+            pass
+
+    def clean_preferences(self):
+        """Removed preferences added by mozrunner."""
+        for filename in self.written_prefs:
+            if not os.path.exists(os.path.join(self.profile, filename)):
+                # file has been deleted
+                break
+            while True:
+                if not self.pop_preferences(filename):
+                    break
 
     @classmethod
     def clone(cls, path_from, path_to=None, **kwargs):
@@ -139,10 +195,9 @@ class Profile(object):
         c.__del__ = c.cleanup = types.MethodType(cleanup_clone(cls.cleanup), c)
         return c
 
-    def create_new_profile(self):
-        """Create a new clean temporary profile which is a simple empty folder"""
-        return tempfile.mkdtemp(suffix='.mozrunner')
-
+    def exists(self):
+        """returns whether the profile exists or not"""
+        return os.path.exists(self.profile)
 
     ### methods for preferences
 
@@ -175,7 +230,9 @@ class Profile(object):
         returns True if popped
         """
 
-        lines = file(os.path.join(self.profile, filename)).read().splitlines()
+        path = os.path.join(self.profile, filename)
+        with file(path) as f:
+            lines = f.read().splitlines()
         def last_index(_list, value):
             """
             returns the last index of an item;
@@ -199,64 +256,10 @@ class Profile(object):
 
         # write the prefs
         cleaned_prefs = '\n'.join(lines[:s] + lines[e+1:])
-        f = file(os.path.join(self.profile, 'user.js'), 'w')
-        f.write(cleaned_prefs)
-        f.close()
+        with file(path, 'w') as f:
+            f.write(cleaned_prefs)
         return True
 
-    def clean_preferences(self):
-        """Removed preferences added by mozrunner."""
-        for filename in self.written_prefs:
-            if not os.path.exists(os.path.join(self.profile, filename)):
-                # file has been deleted
-                break
-            while True:
-                if not self.pop_preferences(filename):
-                    break
-
-    ### cleanup
-
-    def _cleanup_error(self, function, path, excinfo):
-        """ Specifically for windows we need to handle the case where the windows
-            process has not yet relinquished handles on files, so we do a wait/try
-            construct and timeout if we can't get a clear road to deletion
-        """
-
-        try:
-            from exceptions import WindowsError
-            from time import sleep
-            def is_file_locked():
-                return excinfo[0] is WindowsError and excinfo[1].winerror == 32
-
-            if excinfo[0] is WindowsError and excinfo[1].winerror == 32:
-                # Then we're on windows, wait to see if the file gets unlocked
-                # we wait 10s
-                count = 0
-                while count < 10:
-                    sleep(1)
-                    try:
-                        function(path)
-                        break
-                    except:
-                        count += 1
-        except ImportError:
-            # We can't re-raise an error, so we'll hope the stuff above us will throw
-            pass
-
-    def cleanup(self):
-        """Cleanup operations for the profile."""
-        if self.restore:
-            if self.create_new:
-                if os.path.exists(self.profile):
-                    rmtree(self.profile, onerror=self._cleanup_error)
-            else:
-                self.clean_preferences()
-                self.addon_manager.clean_addons()
-                self.permissions.clean_db()
-                self.webapps.clean()
-
-    __del__ = cleanup
-
     ### methods for introspection
 
     def summary(self, return_parts=False):
